==> transmedika_one_to_one_call-1.0.6.tar.gz: lib/webrtc/one_to_one/usecase/one_to_one_usecase.dart <==
import 'dart:convert';

import 'package:flutter/foundation.dart';
import 'package:flutter_webrtc/flutter_webrtc.dart';
import 'package:transmedika_one_to_one_call/webrtc/one_to_one/models/consultation.dart';
import 'package:transmedika_one_to_one_call/webrtc/one_to_one/models/params/notify_caller_param.dart';
import 'package:transmedika_one_to_one_call/webrtc/one_to_one/models/params/reject_param.dart';
import 'package:transmedika_one_to_one_call/webrtc/one_to_one/models/registration_session.dart';

import '../../../models/response/candidate.dart';
import '../../../models/response/socket_response.dart';
import '../../../usecase/socket_usecase.dart';
import '../../constants.dart';
import '../models/params/call_param.dart';
import '../models/params/ice_candidate_param.dart';
import '../models/params/incoming_call_param.dart';
import '../models/params/register_param.dart';
import '../models/params/stop_param.dart';
import 'one_to_one_usecase_service.dart';

class OneToOneUseCase extends SocketUseCase implements OneToOneUseCaseService{
  late RTCPeerConnection _peerConnection;
  late MediaStream? _localStream;
  final List<RTCIceCandidate> remoteCandidates = <RTCIceCandidate>[];
  final List<RTCRtpSender> _senders = <RTCRtpSender>[];
  final List<MediaStream> _remoteStreams = <MediaStream>[];

  @override
  Future<void> register({required String from, required String to, required bool isCalling}) async {
    RegisterParam param = RegisterParam(
      id: Constants.register,
      from: from,
      to: to,
      isCalling: isCalling
    );
    await getSocket().send(const JsonEncoder().convert(param));
  }

  @override
  Future<void> createLocalPeerConnection({
    required Function(MediaStream stream) onLocalStream,
    required Function(MediaStream stream) onAddRemoteStream,
    required Function(MediaStream stream)? onRemoveRemoteStream
  }) async {
    _localStream = await createStream();
    onLocalStream.call(_localStream!);
    _peerConnection = await createPeerConnection({
      ...Constants.iceServers,
      ...Constants.rtcConfig
    }, Constants.constraints);

    _peerConnection.onIceCandidate = (candidate) {
      onIceCandidate(candidate);
    };

    switch (Constants.sdpSemantics) {
      case Constants.planB:
        _peerConnection.onAddStream = (MediaStream stream) {
          onAddRemoteStream.call(stream);
          _remoteStreams.add(stream);
        };
        await _peerConnection.addStream(_localStream!);
        break;
      case Constants.unifiedPlan:
        _peerConnection.onTrack = (event) {
          if (event.track.kind == 'video') {
            onAddRemoteStream.call(event.streams[0]);
          }
        };

        _localStream?.getTracks().forEach((track) async {
          _senders.add(await _peerConnection.addTrack(track, _localStream!));
        });

        break;
    }

    _peerConnection.onRemoveStream = (stream) {
      onRemoveRemoteStream?.call(stream);
      _remoteStreams.removeWhere((it) {
        return (it.id == stream.id);
      });
    };
  }

  @override
  Future<MediaStream> createStream() async {
    MediaStream stream = await navigator.mediaDevices.getUserMedia(Constants.mediaConstraints);
    return stream;
  }

  @override
  RTCPeerConnection? getPeerConnection() {
    return _peerConnection;
  }

  @override
  Future<void> stop() async {
    StopParam param = StopParam(
        id: Constants.stop
    );
    await getSocket().send(const JsonEncoder().convert(param));
  }

  @override
  Future<void> publishCallVideo({
    required RegistrationSession from,
    required RegistrationSession to,
    required RTCSessionDescription sessionDescription}) async {
    CallParam param = CallParam(
      id: Constants.call,
      to: to,
      from: from,
      sdpOffer: sessionDescription.sdp
    );
    await getSocket().send(const JsonEncoder().convert(param));
  }

  @override
  Future<void> publishIncomingCallVideo({
    required RegistrationSession to,
    required RTCSessionDescription sessionDescription,
    Consultation? consultation
  }) async {
    IncomingCallParam param = IncomingCallParam(
      id: Constants.incomingCallResponse,
      from: to,
      callResponse: Constants.accept,
      consultation: consultation!,
      sdpOffer: sessionDescription.sdp
    );
    await getSocket().send(const JsonEncoder().convert(param));
  }

  @override
  Future<void> iceCandidate(SocketResponse response) async {
    Candidate? candidateModel = response.candidate;
    RTCIceCandidate iceCandidate = RTCIceCandidate(candidateModel?.sdp, candidateModel?.sdpMid, candidateModel?.sdpMLineIndex);
    RTCSessionDescription? remoteDescription = await _peerConnection.getRemoteDescription();

    switch(_peerConnection.signalingState){
      case RTCSignalingState.RTCSignalingStateClosed:
        debugPrint("saveIceCandidate error: PeerConnection object is closed");
        break;
      case RTCSignalingState.RTCSignalingStateStable:
        if(remoteDescription!=null){
          _peerConnection.addCandidate(iceCandidate);
        }else{
          remoteCandidates.add(iceCandidate);
        }
        break;
      default:
        remoteCandidates.add(iceCandidate);
    }
  }



  @override
  Future<void> receiveVideoAnswer(SocketResponse response) async {
    RTCSessionDescription sessionDescription = RTCSessionDescription(response.sdpAnswer,'answer');
    _peerConnection.setRemoteDescription(sessionDescription);

    if (remoteCandidates.isNotEmpty) {
      for (var candidate in remoteCandidates) {
        await _peerConnection.addCandidate(candidate);
      }
      remoteCandidates.clear();
    }
  }

  @override
  Future<void> onIceCandidate(RTCIceCandidate rtcIceCandidate) async {
    Candidate candidate = Candidate(
        sdpMid: rtcIceCandidate.sdpMid!,
        sdpMLineIndex: rtcIceCandidate.sdpMLineIndex!,
        sdp: rtcIceCandidate.candidate!);

    IceCandidateParam param = IceCandidateParam(
      id: Constants.onIceCandidate,
      candidate: candidate
    );
    await getSocket().send(const JsonEncoder().convert(param));
  }

  @override
  Future<void> createOffer({
    required RegistrationSession from,
    required RegistrationSession to,
    required bool isCalling,
    Consultation? consultation
  }) async {
    try {
      RTCSessionDescription sessionDescription = await _peerConnection.createOffer(Constants.constraints);
      await _peerConnection.setLocalDescription(sessionDescription);
      if(isCalling){
        publishCallVideo(from: from, to: to, sessionDescription: sessionDescription);
      }else {
        publishIncomingCallVideo(to: to, sessionDescription: sessionDescription, consultation: consultation);
      }
    } catch (e) {
      debugPrint(e.toString());
    }
  }

  @override
  Future<void> closeSession() async {
    _localStream?.getTracks().forEach((element) async {
      await element.stop();
    });
    await _localStream?.dispose();
    _localStream = null;

    await _peerConnection.close();
    _senders.clear();
  }

  @override
  Future<void> notifyCaller({required RegistrationSession from, required String to}) async{
    RegistrationSession registrationSessionTo = RegistrationSession(
        name: to,
        sessionId: null
    );
    
    NotifyCallerParam notifyCallerParam = NotifyCallerParam(id: Constants.notifyCaller, from: from, to: registrationSessionTo);
    await getSocket().send(const JsonEncoder().convert(notifyCallerParam));
  }

  @override
  Future<void> reject({required RegistrationSession from, required RegistrationSession to}) async{
    RejectParam param = RejectParam(
        id: Constants.rejectCalling,
        from: from,
        to: to
    );
    await getSocket().send(const JsonEncoder().convert(param));
  }

  @override
  void muteMic() {
    if (_localStream != null) {
      bool enabled = _localStream!.getAudioTracks()[0].enabled;
      _localStream!.getAudioTracks()[0].enabled = !enabled;
    }
  }

  @override
  void switchCamera() {
    if (_localStream != null) {
      Helper.switchCamera(_localStream!.getVideoTracks()[0]);
    }
  }

  @override
  void enableVideo() {
    if (_localStream != null) {
      bool enabled = _localStream!.getVideoTracks()[0].enabled;
      _localStream!.getVideoTracks()[0].enabled = !enabled;
    }
  }
}

==> transmedika_one_to_one_call-1.0.10.tar.gz: lib/webrtc/one_to_one/usecase/one_to_one_usecase.dart <==
import 'dart:convert';

import 'package:flutter/foundation.dart';
import 'package:flutter_webrtc/flutter_webrtc.dart';
import 'package:transmedika_one_to_one_call/webrtc/one_to_one/models/consultation.dart';
import 'package:transmedika_one_to_one_call/webrtc/one_to_one/models/params/notify_caller_param.dart';
import 'package:transmedika_one_to_one_call/webrtc/one_to_one/models/params/reject_param.dart';
import 'package:transmedika_one_to_one_call/webrtc/one_to_one/models/registration_session.dart';

import '../../../usecase/socket_usecase.dart';
import '../../constants.dart';
import '../models/params/call_param.dart';
import '../models/params/ice_candidate_param.dart';
import '../models/params/incoming_call_param.dart';
import '../models/params/register_param.dart';
import '../models/params/stop_param.dart';
import '../models/response/candidate.dart';
import '../models/response/socket_response.dart';
import 'one_to_one_usecase_service.dart';

class OneToOneUseCase extends SocketUseCase implements OneToOneUseCaseService{
  late RTCPeerConnection _peerConnection;
  late MediaStream? _localStream;
  final List<RTCIceCandidate> remoteCandidates = <RTCIceCandidate>[];
  final List<RTCRtpSender> _senders = <RTCRtpSender>[];
  final List<MediaStream> _remoteStreams = <MediaStream>[];

  @override
  Future<void> register({required String from, required String to, required bool isCalling}) async {
    RegisterParam param = RegisterParam(
      id: Constants.register,
      from: from,
      to: to,
      isCalling: isCalling
    );
    await getSocket().send(const JsonEncoder().convert(param));
  }

  @override
  Future<void> createLocalPeerConnection({
    required Function(MediaStream stream) onLocalStream,
    required Function(MediaStream stream) onAddRemoteStream,
    required Function(MediaStream stream)? onRemoveRemoteStream
  }) async {
    _localStream = await createStream();
    onLocalStream.call(_localStream!);
    _peerConnection = await createPeerConnection({
      ...Constants.iceServers,
      ...Constants.rtcConfig
    }, Constants.constraints);

    _peerConnection.onIceCandidate = (candidate) {
      onIceCandidate(candidate);
    };

    switch (Constants.sdpSemantics) {
      case Constants.planB:
        _peerConnection.onAddStream = (MediaStream stream) {
          onAddRemoteStream.call(stream);
          _remoteStreams.add(stream);
        };
        await _peerConnection.addStream(_localStream!);
        break;
      case Constants.unifiedPlan:
        _peerConnection.onTrack = (event) {
          if (event.track.kind == 'video') {
            onAddRemoteStream.call(event.streams[0]);
          }
        };

        _localStream?.getTracks().forEach((track) async {
          _senders.add(await _peerConnection.addTrack(track, _localStream!));
        });

        break;
    }

    _peerConnection.onRemoveStream = (stream) {
      onRemoveRemoteStream?.call(stream);
      _remoteStreams.removeWhere((it) {
        return (it.id == stream.id);
      });
    };
  }

  @override
  Future<MediaStream> createStream() async {
    MediaStream stream = await navigator.mediaDevices.getUserMedia(Constants.mediaConstraints);
    return stream;
  }

  @override
  RTCPeerConnection? getPeerConnection() {
    return _peerConnection;
  }

  @override
  Future<void> stop() async {
    StopParam param = StopParam(
        id: Constants.stop
    );
    await getSocket().send(const JsonEncoder().convert(param));
  }

  @override
  Future<void> publishCallVideo({
    required RegistrationSession from,
    required RegistrationSession to,
    required RTCSessionDescription sessionDescription}) async {
    CallParam param = CallParam(
      id: Constants.call,
      to: to,
      from: from,
      sdpOffer: sessionDescription.sdp
    );
    await getSocket().send(const JsonEncoder().convert(param));
  }

  @override
  Future<void> publishIncomingCallVideo({
    required RegistrationSession to,
    required RTCSessionDescription sessionDescription,
    Consultation? consultation
  }) async {
    IncomingCallParam param = IncomingCallParam(
      id: Constants.incomingCallResponse,
      from: to,
      callResponse: Constants.accept,
      consultation: consultation!,
      sdpOffer: sessionDescription.sdp
    );
    await getSocket().send(const JsonEncoder().convert(param));
  }

  @override
  Future<void> iceCandidate(SocketResponse response) async {
    Candidate? candidateModel = response.candidate;
    RTCIceCandidate iceCandidate = RTCIceCandidate(candidateModel?.sdp, candidateModel?.sdpMid, candidateModel?.sdpMLineIndex);
    RTCSessionDescription? remoteDescription = await _peerConnection.getRemoteDescription();

    switch(_peerConnection.signalingState){
      case RTCSignalingState.RTCSignalingStateClosed:
        debugPrint("saveIceCandidate error: PeerConnection object is closed");
        break;
      case RTCSignalingState.RTCSignalingStateStable:
        if(remoteDescription!=null){
          _peerConnection.addCandidate(iceCandidate);
        }else{
          remoteCandidates.add(iceCandidate);
        }
        break;
      default:
        remoteCandidates.add(iceCandidate);
    }
  }



  @override
  Future<void> receiveVideoAnswer(SocketResponse response) async {
    RTCSessionDescription sessionDescription = RTCSessionDescription(response.sdpAnswer,'answer');
    _peerConnection.setRemoteDescription(sessionDescription);

    if (remoteCandidates.isNotEmpty) {
      for (var candidate in remoteCandidates) {
        await _peerConnection.addCandidate(candidate);
      }
      remoteCandidates.clear();
    }
  }

  @override
  Future<void> onIceCandidate(RTCIceCandidate rtcIceCandidate) async {
    Candidate candidate = Candidate(
        sdpMid: rtcIceCandidate.sdpMid!,
        sdpMLineIndex: rtcIceCandidate.sdpMLineIndex!,
        sdp: rtcIceCandidate.candidate!);

    IceCandidateParam param = IceCandidateParam(
      id: Constants.onIceCandidate,
      candidate: candidate
    );
    await getSocket().send(const JsonEncoder().convert(param));
  }

  @override
  Future<void> createOffer({
    required RegistrationSession from,
    required RegistrationSession to,
    required bool isCalling,
    Consultation? consultation
  }) async {
    try {
      RTCSessionDescription sessionDescription = await _peerConnection.createOffer(Constants.constraints);
      await _peerConnection.setLocalDescription(sessionDescription);
      if(isCalling){
        publishCallVideo(from: from, to: to, sessionDescription: sessionDescription);
      }else {
        publishIncomingCallVideo(to: to, sessionDescription: sessionDescription, consultation: consultation);
      }
    } catch (e) {
      debugPrint(e.toString());
    }
  }

  @override
  Future<void> closeSession() async {
    _localStream?.getTracks().forEach((element) async {
      await element.stop();
    });
    await _localStream?.dispose();
    _localStream = null;

    await _peerConnection.close();
    _senders.clear();
  }

  @override
  Future<void> notifyCaller({required RegistrationSession from, required String to}) async{
    RegistrationSession registrationSessionTo = RegistrationSession(
        name: to,
        sessionId: null
    );
    
    NotifyCallerParam notifyCallerParam = NotifyCallerParam(id: Constants.notifyCaller, from: from, to: registrationSessionTo);
    await getSocket().send(const JsonEncoder().convert(notifyCallerParam));
  }

  @override
  Future<void> reject({required RegistrationSession from, required RegistrationSession to}) async{
    RejectParam param = RejectParam(
        id: Constants.rejectCalling,
        from: from,
        to: to
    );
    await getSocket().send(const JsonEncoder().convert(param));
  }

  @override
  void muteMic() {
    if (_localStream != null) {
      bool enabled = _localStream!.getAudioTracks()[0].enabled;
      _localStream!.getAudioTracks()[0].enabled = !enabled;
    }
  }

  @override
  void switchCamera() {
    if (_localStream != null) {
      Helper.switchCamera(_localStream!.getVideoTracks()[0]);
    }
  }

  @override
  void enableVideo() {
    if (_localStream != null) {
      bool enabled = _localStream!.getVideoTracks()[0].enabled;
      _localStream!.getVideoTracks()[0].enabled = !enabled;
    }
  }
}
